@@ -1,5 +1,5 @@
-#include <CombPermUtils.h>
-#include <CountGmp.h>
+#include "CombPermUtils.h"
+#include "CountGmp.h"
 
 std::vector<int> nonZeroVec(std::vector<int> v) {
     std::vector<int> nonZero;
@@ -11,23 +11,21 @@ std::vector<int> nonZeroVec(std::vector<int> v) {
     return nonZero;
 }
 
-std::vector<int> nthPermutation(int n, int r, double myIndex, bool isRep, bool isMult,
-                                std::vector<int> Reps, std::vector<int> freqs, bool isStarter = false) {
+std::vector<int> nthPermutation(const int n, const int r, double myIndex, bool isRep, 
+                                bool isMult, std::vector<int> Reps, std::vector<int> freqs,
+                                bool isStarter = false) {
     
-    int j = 0, n1 = n;
     double temp, index1 = myIndex;
-    std::vector<int> res(r);
+    std::vector<int> res(r); 
     
     if (isMult) {
         
         ++index1;
         std::vector<int> Counts;
-        int r1 = r - 1;
         double test, index2 = index1;
         
-        for (int k = 0; k < r; ++k, --r1) {
-            
-            j = 0;
+        for (int k = 0, r1 = r - 1; k < r; ++k, --r1) {
+            int j = 0;
             while (Reps[j] == 0)
                 ++j;
             
@@ -35,7 +33,7 @@ std::vector<int> nthPermutation(int n, int r, double myIndex, bool isRep, bool i
             Counts = nonZeroVec(Reps);
             test = temp = MultisetPermRowNum(Counts.size(), r1, Counts);
             
-            while (test < index1) {
+            for (; test < index1; test += temp) {
                 index2 -= temp;
                 ++Reps[j];
                 ++j;
@@ -44,10 +42,8 @@ std::vector<int> nthPermutation(int n, int r, double myIndex, bool isRep, bool i
                     ++j;
                 
                 --Reps[j];
-                
                 Counts = nonZeroVec(Reps);
                 temp = MultisetPermRowNum(Counts.size(), r1, Counts);
-                test += temp;
             }
             
             res[k] = j;
@@ -68,22 +64,23 @@ std::vector<int> nthPermutation(int n, int r, double myIndex, bool isRep, bool i
                 res.push_back(freqs[i]);
         }
     } else if (isRep) {
-        temp = std::pow((double) n, (double) r);
+        temp = std::pow(static_cast<double>(n), static_cast<double>(r));
         
         for (int k = 0; k < r; ++k) {
             temp /= n;
-            j = (int) std::trunc(index1 / temp);
+            int j = static_cast<int>(index1 / temp);
             res[k] = j;
             index1 -= (temp * j);
         }
+        
     } else {
         temp = NumPermsNoRep(n, r);
         std::vector<int> indexVec(n);
         std::iota(indexVec.begin(), indexVec.end(), 0);
         
-        for (int k = 0; k < r; ++k, --n1) {
+        for (int k = 0, n1 = n; k < r; ++k, --n1) {
             temp /= n1;
-            j = (int) std::trunc(index1 / temp);
+            int j = static_cast<int>(index1 / temp);
             res[k] = indexVec[j];
             index1 -= (temp * j);
             indexVec.erase(indexVec.begin() + j);
@@ -109,18 +106,17 @@ std::vector<int> nthPermutation(int n, int r, double myIndex, bool isRep, bool i
     return res;
 }
 
-std::vector<int> nthCombination(int n, int r, double myIndex, bool isRep,
+std::vector<int> nthCombination(const int n, const int r, double myIndex, bool isRep, 
                                 bool isMult, std::vector<int> Reps) {
     
-    int j = 0, n1 = n, r1 = r - 1, rTemp;
-    double test, temp, index1 = myIndex, index2 = myIndex;
+    double test, temp;
+    double index1 = myIndex, index2 = myIndex;
     std::vector<int> res(r);
-    unsigned long int uR = r;
     
     if (isMult) {
         std::vector<int> Counts = Reps;
         
-        for (int k = 0; k < r; ++k, --r1) {
+        for (int k = 0, j = 0, n1 = n, r1 = r - 1; k < r; ++k, --r1) {
             
             --Counts[0];
             if (Counts[0] == 0 && Counts.size() > 1) {
@@ -128,13 +124,13 @@ std::vector<int> nthCombination(int n, int r, double myIndex, bool isRep,
                 Counts.erase(Counts.begin());
             }
             
-            test = temp = MultisetCombRowNum(n1, r1, Counts);
+            test = temp = MultisetCombRowNumFast(n1, r1, Counts);
             
-            while (test <= index1) {
+            for (; test <= index1; ++j, test += temp) {
                 index2 -= temp;
                 Reps[j] = 0;
                 
-                if ((int) Counts.size() == (n - j)) {
+                if (static_cast<int>(Counts.size()) == (n - j)) {
                     --n1;
                     Counts.erase(Counts.begin());
                 }
@@ -145,52 +141,51 @@ std::vector<int> nthCombination(int n, int r, double myIndex, bool isRep,
                     Counts.erase(Counts.begin());
                 }
                 
-                ++j;
-                temp = MultisetCombRowNum(n1, r1, Counts);
-                test += temp;
+                temp = MultisetCombRowNumFast(n1, r1, Counts);
             }
             
             res[k] = j;
             index1 = index2;
             
             --Reps[j];
-            if (Reps[j] <= 0)
-                ++j;
+            if (Reps[j] <= 0) ++j;
         }
     } else if (isRep) {
         
-        for (std::size_t k = 0; k < uR; ++k, --r1) {
-            temp = test = NumCombsWithRep(n1, r1);
-            rTemp = n1 - 1;
-            while (test <= index1) {
+        temp = NumCombsWithRep(n, r - 1);
+        
+        for (int k = 0, j = 0, n1 = n, r1 = r - 1; k < r; ++k, --r1) {
+            test = temp;
+            
+            for (; test <= index1; --n1, ++j, test += temp) {
                 index2 -= temp;
-                temp *= rTemp;
+                temp *= (n1 - 1);
                 temp /= (n1 + r1 - 1);
-                --n1;
-                ++j;
-                --rTemp;
-                test += temp;
             }
+            
+            temp *= r1;
+            temp /= (n1 + r1 - 1);
             res[k] = j;
             index1 = index2;
         }
         
     } else {
         
-        --n1;
+        temp = nChooseK(n - 1, r - 1);
         
-        for (std::size_t k = 0; k < uR; ++k, --n1, --r1, ++j) {
-            temp = test = nChooseK(n1, r1);
-            rTemp = n1 - r1;
-            while (test <= index1) {
+        for (int k = 0, j = 0, n1 = n - 1, r1 = r - 1; 
+                                k < r; ++k, --n1, --r1, ++j) {
+            test = temp;
+            
+            for (int rTemp = n1 - r1; test <= index1; 
+                        --n1, ++j, --rTemp, test += temp) {
                 index2 -= temp;
                 temp *= rTemp;
                 temp /= n1;
-                --n1;
-                ++j;
-                --rTemp;
-                test += temp;
             }
+            
+            temp *= r1;
+            temp /= n1;
             res[k] = j;
             index1 = index2;
         }
@@ -199,9 +194,10 @@ std::vector<int> nthCombination(int n, int r, double myIndex, bool isRep,
     return res;
 }
 
-std::vector<int> nthPermutationGmp(int n, int r, mpz_t myIndex, bool isRep, bool isMult,
-                                   std::vector<int> Reps, std::vector<int> freqs, bool isStarter = false) {
-    int j = 0, n1 = n;
+std::vector<int> nthPermutationGmp(const int n, const int r, mpz_t myIndex, bool isRep, bool isMult,
+                                   std::vector<int> Reps, std::vector<int> freqs,
+                                   bool isStarter = false) {
+    
     mpz_t temp, temp2, index1;
     mpz_init(temp); mpz_init(temp2); mpz_init(index1);
     mpz_set(index1, myIndex);
@@ -211,20 +207,19 @@ std::vector<int> nthPermutationGmp(int n, int r, mpz_t myIndex, bool isRep, bool
         
         mpz_add_ui(index1, index1, 1);
         std::vector<int> Counts;
-        int r1 = r - 1;
         mpz_t test, index2;
         mpz_init(test); mpz_init(index2);
         mpz_set(index2, index1);
         
-        for (int k = 0; k < r; ++k, --r1) {
+        for (int k = 0, r1 = r - 1; k < r; ++k, --r1) {
             
-            j = 0;
+            int j = 0;
             while (Reps[j] == 0)
                 ++j;
             
             --Reps[j];
             Counts = nonZeroVec(Reps);
-            MultisetPermRowNumGmp(temp, (int) Counts.size(), r1, Counts);
+            MultisetPermRowNumGmp(temp, static_cast<int>(Counts.size()), r1, Counts);
             mpz_set(test, temp);
             
             while (mpz_cmp(test, index1) < 0) {
@@ -236,9 +231,8 @@ std::vector<int> nthPermutationGmp(int n, int r, mpz_t myIndex, bool isRep, bool
                     ++j;
 
                 --Reps[j];
-                
                 Counts = nonZeroVec(Reps);
-                MultisetPermRowNumGmp(temp, (int) Counts.size(), r1, Counts);
+                MultisetPermRowNumGmp(temp, static_cast<int>(Counts.size()), r1, Counts);
                 mpz_add(test, test, temp);
             }
             
@@ -268,20 +262,21 @@ std::vector<int> nthPermutationGmp(int n, int r, mpz_t myIndex, bool isRep, bool
         for (int k = 0; k < r; ++k) {
             mpz_divexact_ui(temp, temp, n);
             mpz_tdiv_q(temp2, index1, temp);
-            j = mpz_get_si(temp2);
+            int j = mpz_get_si(temp2);
             res[k] = j;
             mpz_submul_ui(index1, temp, j);
         }
+        
     } else {
-        mpz_set_ui(temp, 1);
+        mpz_set_ui(temp, 1u);
         NumPermsNoRepGmp(temp, n, r);
         std::vector<int> indexVec(n);
         std::iota(indexVec.begin(), indexVec.end(), 0);
         
-        for (int k = 0; k < r; ++k, --n1) {
+        for (int k = 0, n1 = n; k < r; ++k, --n1) {
             mpz_divexact_ui(temp, temp, n1);
             mpz_tdiv_q(temp2, index1, temp);
-            j = mpz_get_si(temp2);
+            int j = mpz_get_si(temp2);
             res[k] = indexVec[j];
             mpz_submul_ui(index1, temp, j);
             indexVec.erase(indexVec.begin() + j);
@@ -308,24 +303,20 @@ std::vector<int> nthPermutationGmp(int n, int r, mpz_t myIndex, bool isRep, bool
     return res;
 }
 
-std::vector<int> nthCombinationGmp(int n, int r, mpz_t myIndex, bool isRep,
+std::vector<int> nthCombinationGmp(const int n, const int r, mpz_t myIndex, bool isRep,
                                    bool isMult, std::vector<int> Reps) {
-    
-    int j = 0, n1 = n, r1 = r - 1, rTemp;
+    ;
     mpz_t test, temp, index1, index2;
     mpz_init(test); mpz_init(temp);
     mpz_init(index1); mpz_init(index2);
     mpz_set(index1, myIndex);
     mpz_set(index2, myIndex);
-    
     std::vector<int> res(r);
-    unsigned long int uR = r;
     
     if (isMult) {
-        
         std::vector<int> Counts = Reps;
         
-        for (int k = 0; k < r; ++k, --r1) {
+        for (int k = 0, n1 = n, j = 0, r1 = r - 1; k < r; ++k, --r1) {
             
             --Counts[0];
             if (Counts[0] == 0 && Counts.size() > 1) {
@@ -336,11 +327,11 @@ std::vector<int> nthCombinationGmp(int n, int r, mpz_t myIndex, bool isRep,
             MultisetCombRowNumGmp(temp, n1, r1, Counts);
             mpz_set(test, temp);
             
-            while (mpz_cmp(test, index1) <= 0) {
+            for (; mpz_cmp(test, index1) <= 0; ++j) {
                 mpz_sub(index2, index2, temp);
                 Reps[j] = 0;
                 
-                if ((int) Counts.size() == (n - j)) {
+                if (static_cast<int>(Counts.size()) == (n - j)) {
                     --n1;
                     Counts.erase(Counts.begin());
                 }
@@ -351,8 +342,6 @@ std::vector<int> nthCombinationGmp(int n, int r, mpz_t myIndex, bool isRep,
                     Counts.erase(Counts.begin());
                 }
                 
-                ++j;
-                
                 MultisetCombRowNumGmp(temp, n1, r1, Counts);
                 mpz_add(test, test, temp);
             }
@@ -361,48 +350,48 @@ std::vector<int> nthCombinationGmp(int n, int r, mpz_t myIndex, bool isRep,
             mpz_set(index1, index2);
             
             --Reps[j];
-            if (Reps[j] <= 0)
-                ++j;
+            if (Reps[j] <= 0) ++j;
         }
+        
     } else if (isRep) {
         
-        for (std::size_t k = 0; k < uR; ++k, --r1) {
-            NumCombsWithRepGmp(temp, n1, r1);
+        NumCombsWithRepGmp(temp, n, r - 1);
+        
+        for (int k = 0, j = 0, n1 = n, r1 = r - 1; k < r; ++k, --r1) {
             mpz_set(test, temp);
-            rTemp = n1 - 1;
             
-            while (mpz_cmp(test, index1) <= 0) {
+            for (; mpz_cmp(test, index1) <= 0; ++j, --n1) {
                 mpz_sub(index2, index2, temp);
-                mpz_mul_ui(temp, temp, rTemp);
+                mpz_mul_ui(temp, temp, n1 - 1);
                 mpz_divexact_ui(temp, temp, n1 + r1 - 1);
-                --n1;
-                ++j;
-                --rTemp;
                 mpz_add(test, test, temp);
             }
+            
+            mpz_mul_ui(temp, temp, r1);
+            if ((n1 + r1) > 1) mpz_divexact_ui(temp, temp, n1 + r1 - 1);
             res[k] = j;
             mpz_set(index1, index2);
         }
         
     } else {
         
-        --n1;
+        nChooseKGmp(temp, n - 1, r - 1);
         
-        for (std::size_t k = 0; k < uR; ++k, --n1, --r1, ++j) {
-            nChooseKGmp(temp, n1, r1);
+        for (int k = 0, j = 0, n1 = n - 1, r1 = r - 1; 
+                            k < r; ++k, --n1, --r1, ++j) {
             mpz_set(test, temp);
-            rTemp = n1 - r1;
             
-            while (mpz_cmp(test, index1) <= 0) {
+            for (int rTemp = n1 - r1; 
+                 mpz_cmp(test, index1) <= 0; --rTemp, ++j, --n1) {
+                
                 mpz_sub(index2, index2, temp);
                 mpz_mul_ui(temp, temp, rTemp);
                 mpz_divexact_ui(temp, temp, n1);
-                --n1;
-                ++j;
-                --rTemp;
                 mpz_add(test, test, temp);
             }
             
+            mpz_mul_ui(temp, temp, r1);
+            if (n1 > 0) mpz_divexact_ui(temp, temp, n1);
             res[k] = j;
             mpz_set(index1, index2);
         }
@@ -410,6 +399,5 @@ std::vector<int> nthCombinationGmp(int n, int r, mpz_t myIndex, bool isRep,
     
     mpz_clear(index1); mpz_clear(index2);
     mpz_clear(temp); mpz_clear(test);
-    
     return res;
 }
