@@ -1,7 +1,7 @@
 context("testing divisorsRcpp")
 
 test_that("divisorsRcpp generates correct numbers", {
-    options(scipen = 999)
+    options(scipen = 50)
     expect_equal(divisorsRcpp(0), numeric(0))
     expect_equal(length(divisorsRcpp(1:100)), 100)
     expect_equal(divisorsRcpp(2), c(1, 2))
@@ -22,6 +22,8 @@ test_that("divisorsRcpp generates correct numbers", {
                         div
                      }))
     
+    expect_equal(divisorsRcpp((1e6):(1e6 + 1e3)), divisorsSieve(1e6, 1e6 + 1e3))
+    
     expect_equal(divisorsRcpp(-1*1e10), c(-1* rev(divisorsRcpp(1e10)), divisorsRcpp(1e10)))
     
     expect_equal(divisorsRcpp(1000), c(1,2,4,5,8,10,20,
@@ -35,6 +37,11 @@ test_that("divisorsRcpp generates correct numbers", {
     expect_equal(as.integer(names(divisorsRcpp(100, namedList = TRUE))), integer(0))
     expect_equal(as.numeric(names(divisorsRcpp((10^12):(10^12 + 100),
                                                  namedList = TRUE))), (10^12):(10^12 + 100))
+    
+    ## Test Parallel
+    mySample = sample(10^9, 20)
+    expect_equal(divisorsRcpp(mySample), divisorsRcpp(mySample, nThreads = 2))
+    options(scipen = 0)
 })
 
 test_that("divisorsRcpp produces appropriate error messages", {
@@ -42,5 +49,7 @@ test_that("divisorsRcpp produces appropriate error messages", {
     expect_error(divisorsRcpp(-2^53), "each element must be less than")
     expect_error(divisorsRcpp("10"), "must be of type numeric or integer")
     expect_error(divisorsRcpp(c(-2^53, 1:100)), "the abs value of each element must be less than")
-    expect_error(divisorsRcpp(100, namedList = "TRUE"), "Not compatible with requested type")
+    expect_error(divisorsRcpp(100, namedList = "TRUE"), 
+                 "Only logical values are supported for namedList")
+    expect_error(divisorsRcpp(100:200, nThreads = "9"), "must be of type numeric or integer")
 })
